@@ -36,32 +36,30 @@ class UDPPacketInternal : public UDPPacket
 {
 public:
   UDPPacketInternal();
-  virtual ~UDPPacketInternal();
+  ~UDPPacketInternal() override;
 
-  void append_block_internal(IOBufferBlock *block);
-
-  virtual void free();
+  void free() override;
 
   SLINK(UDPPacketInternal, alink); // atomic link
   // packet scheduling stuff: keep it a doubly linked list
-  uint64_t pktLength;
+  uint64_t pktLength = 0;
 
-  int reqGenerationNum;
-  ink_hrtime delivery_time; // when to deliver packet
+  int reqGenerationNum     = 0;
+  ink_hrtime delivery_time = 0; // when to deliver packet
 
   Ptr<IOBufferBlock> chain;
-  Continuation *cont;          // callback on error
-  UDPConnectionInternal *conn; // connection where packet should be sent to.
+  Continuation *cont          = nullptr; // callback on error
+  UDPConnectionInternal *conn = nullptr; // connection where packet should be sent to.
 
-  int in_the_priority_queue;
-  int in_heap;
+  int in_the_priority_queue = 0;
+  int in_heap               = 0;
 };
 
-inkcoreapi extern ClassAllocator<UDPPacketInternal> udpPacketAllocator;
+extern ClassAllocator<UDPPacketInternal> udpPacketAllocator;
 
 TS_INLINE
 UDPPacketInternal::UDPPacketInternal()
-  : pktLength(0), reqGenerationNum(0), delivery_time(0), cont(nullptr), conn(nullptr), in_the_priority_queue(0), in_heap(0)
+
 {
   memset(&from, '\0', sizeof(from));
   memset(&to, '\0', sizeof(to));
@@ -86,7 +84,7 @@ UDPPacketInternal::free()
 TS_INLINE void
 UDPPacket::append_block(IOBufferBlock *block)
 {
-  UDPPacketInternal *p = (UDPPacketInternal *)this;
+  UDPPacketInternal *p = static_cast<UDPPacketInternal *>(this);
 
   if (block) {
     if (p->chain) { // append to end
@@ -104,7 +102,7 @@ UDPPacket::append_block(IOBufferBlock *block)
 TS_INLINE int64_t
 UDPPacket::getPktLength() const
 {
-  UDPPacketInternal *p = (UDPPacketInternal *)this;
+  UDPPacketInternal *p = const_cast<UDPPacketInternal *>(static_cast<const UDPPacketInternal *>(this));
   IOBufferBlock *b;
 
   p->pktLength = 0;
@@ -119,13 +117,13 @@ UDPPacket::getPktLength() const
 TS_INLINE void
 UDPPacket::free()
 {
-  ((UDPPacketInternal *)this)->free();
+  static_cast<UDPPacketInternal *>(this)->free();
 }
 
 TS_INLINE void
 UDPPacket::setContinuation(Continuation *c)
 {
-  ((UDPPacketInternal *)this)->cont = c;
+  static_cast<UDPPacketInternal *>(this)->cont = c;
 }
 
 TS_INLINE void
@@ -138,7 +136,7 @@ UDPPacket::setConnection(UDPConnection *c)
      assert will prevent that.  The "if" clause enables correct
      handling of the connection ref. counts in such a scenario. */
 
-  UDPConnectionInternal *&conn = ((UDPPacketInternal *)this)->conn;
+  UDPConnectionInternal *&conn = static_cast<UDPPacketInternal *>(this)->conn;
 
   if (conn) {
     if (conn == c)
@@ -146,67 +144,25 @@ UDPPacket::setConnection(UDPConnection *c)
     conn->Release();
     conn = nullptr;
   }
-  conn = (UDPConnectionInternal *)c;
+  conn = static_cast<UDPConnectionInternal *>(c);
   conn->AddRef();
 }
 
 TS_INLINE IOBufferBlock *
-UDPPacket::getIOBlockChain(void)
+UDPPacket::getIOBlockChain()
 {
   ink_assert(dynamic_cast<UDPPacketInternal *>(this) != nullptr);
-  return ((UDPPacketInternal *)this)->chain.get();
+  return static_cast<UDPPacketInternal *>(this)->chain.get();
 }
 
 TS_INLINE UDPConnection *
-UDPPacket::getConnection(void)
+UDPPacket::getConnection()
 {
-  return ((UDPPacketInternal *)this)->conn;
+  return static_cast<UDPPacketInternal *>(this)->conn;
 }
 
 TS_INLINE UDPPacket *
-new_UDPPacket(struct sockaddr const *to, ink_hrtime when, char *buf, int len)
-{
-  UDPPacketInternal *p = udpPacketAllocator.alloc();
-
-  p->in_the_priority_queue = 0;
-  p->in_heap               = 0;
-  p->delivery_time         = when;
-  ats_ip_copy(&p->to, to);
-
-  if (buf) {
-    IOBufferBlock *body = new_IOBufferBlock();
-    body->alloc(iobuffer_size_to_index(len));
-    memcpy(body->end(), buf, len);
-    body->fill(len);
-    p->append_block(body);
-  }
-
-  return p;
-}
-
-TS_INLINE UDPPacket *
-new_UDPPacket(struct sockaddr const *to, ink_hrtime when, IOBufferBlock *buf, int len)
-{
-  (void)len;
-  UDPPacketInternal *p = udpPacketAllocator.alloc();
-  IOBufferBlock *body;
-
-  p->in_the_priority_queue = 0;
-  p->in_heap               = 0;
-  p->delivery_time         = when;
-  ats_ip_copy(&p->to, to);
-
-  while (buf) {
-    body = buf->clone();
-    p->append_block(body);
-    buf = buf->next.get();
-  }
-
-  return p;
-}
-
-TS_INLINE UDPPacket *
-new_UDPPacket(struct sockaddr const *to, ink_hrtime when, Ptr<IOBufferBlock> buf)
+new_UDPPacket(struct sockaddr const *to, ink_hrtime when, Ptr<IOBufferBlock> &buf)
 {
   UDPPacketInternal *p = udpPacketAllocator.alloc();
 
@@ -220,13 +176,7 @@ new_UDPPacket(struct sockaddr const *to, ink_hrtime when, Ptr<IOBufferBlock> buf
 }
 
 TS_INLINE UDPPacket *
-new_UDPPacket(ink_hrtime when, Ptr<IOBufferBlock> buf)
-{
-  return new_UDPPacket(nullptr, when, buf);
-}
-
-TS_INLINE UDPPacket *
-new_incoming_UDPPacket(struct sockaddr *from, char *buf, int len)
+new_incoming_UDPPacket(struct sockaddr *from, struct sockaddr *to, Ptr<IOBufferBlock> &block)
 {
   UDPPacketInternal *p = udpPacketAllocator.alloc();
 
@@ -234,12 +184,8 @@ new_incoming_UDPPacket(struct sockaddr *from, char *buf, int len)
   p->in_heap               = 0;
   p->delivery_time         = 0;
   ats_ip_copy(&p->from, from);
-
-  IOBufferBlock *body = new_IOBufferBlock();
-  body->alloc(iobuffer_size_to_index(len));
-  memcpy(body->end(), buf, len);
-  body->fill(len);
-  p->append_block(body);
+  ats_ip_copy(&p->to, to);
+  p->chain = block;
 
   return p;
 }
