@@ -27,7 +27,7 @@
 
 #include "P_RecCore.h"
 
-Diags *diags = NULL;
+Diags *diags = nullptr;
 void RecDumpRecordsHt(RecT rec_type);
 
 //-------------------------------------------------------------------------
@@ -74,7 +74,6 @@ Test01()
 {
   g_config_update_result = 0;
   printf("\n[Test01: Callback Tests]\n");
-  int failures = 0;
 
   printf("  [RecRegisterConfigUpdateCb]\n");
 
@@ -139,7 +138,6 @@ Test02()
 {
   g_config_update_result = 0;
   printf("\n[Test02: Callback (Multi-lock) Test]\n");
-  int failures = 0;
 
   printf("  [RecRegisterConfigUpdateCb]\n");
 
@@ -174,16 +172,16 @@ main(int argc, char **argv)
 {
   // start diags logging
   FILE *log_fp;
-  if ((log_fp = fopen("reclocal.log", "a+")) != NULL) {
-    int status = setvbuf(log_fp, NULL, _IOLBF, 512);
+  if ((log_fp = fopen("reclocal.log", "a+")) != nullptr) {
+    int status = setvbuf(log_fp, nullptr, _IOLBF, 512);
     if (status != 0) {
       fclose(log_fp);
-      log_fp = NULL;
+      log_fp = nullptr;
     }
   }
-  diags = new Diags("rec", NULL, log_fp);
+  diags = new Diags("rec", nullptr, log_fp);
   diags->activate_taglist(diags->base_debug_tags, DiagsTagType_Debug);
-  diags->log(NULL, DTA(DL_Note), "Starting '%s'", argv[0]);
+  diags->log(nullptr, DTA(DL_Note), "Starting '%s'", argv[0]);
 
   // system initialization
   RecLocalInit(diags);
@@ -193,7 +191,7 @@ main(int argc, char **argv)
 
   // test
   Test01(); // Local callbacks
-  Test02(); // Local callbacks -- mulit-lock
+  Test02(); // Local callbacks -- multi-lock
   Test03(); // RecTree
 
   while (true) {
